// Initialisierungstabelle CGRAM
const unsigned char lcd_cgramcode[64] =
  { 0x1f, 0x00, 0x00, 0x04, 0x04, 0x04, 0x04, 0x0e,        // Zeichen 0x00
    0x00, 0x0e, 0x11, 0x04, 0x04, 0x04, 0x04, 0x0e,        // Zeichen 0x01
    0x00, 0x00, 0x0e, 0x0e, 0x04, 0x04, 0x04, 0x0e,        // Zeichen 0x02
    0x00, 0x00, 0x00, 0x04, 0x04, 0x04, 0x04, 0x0e,        // Zeichen 0x03
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x07,        // Zeichen 0x04
    0x00, 0x00, 0x00, 0x00, 0x07, 0x07, 0x07, 0x07,        // Zeichen 0x05
    0x00, 0x00, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07,        // Zeichen 0x06
    0x0e, 0x0a, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00 };      // Zeichen 0x07

// Portpins zu LCD-Display auf Kit
#define     LCD_ON         p4_7
#define     LCD_E          p4_6
#define     LCD_RW         p4_5
#define     LCD_RS         p4_4
// LCD-Daten mit 4-Bit-Interface auf P40..43 (DB4..DB7)

// Verzoegerung in us fuer 16-MHz-Quarz und volle Taktfrequenz
void delay (unsigned int us) {
  while(us) us--;
}

// Zeichenuebergabe an LCD-Display
void lcd_putbyte(unsigned char Ausgabe) {        // RW auf 0; RS nicht bedient
  unsigned char c;
  delay(40);                                     // 40 us warten
  c=Ausgabe/16;
  LCD_E=1; p4&=0xf0; p4|=c; delay(1); LCD_E=0; delay(1);
  c=Ausgabe&0x0f;
  LCD_E=1; p4&=0xf0; p4|=c; delay(1); LCD_E=0;
}

// Init LCD-Display mit "Initialization by instruction"
void lcd_init() {
  unsigned char i;
  delay(20000);                                  // 20 ms nach power-on
  LCD_RS=0; LCD_RW=0;
  LCD_E=1; p4&=0xf3; p4|=0x03; delay(1); LCD_E=0;
  delay(10000);                                  // 30 ms nach power-on
  LCD_E=1; delay(1); LCD_E=0;
  delay(100);                                    // mehr als 100 us...
  LCD_E=1; delay(1); LCD_E=0;                    // damit RESET erledigt
  delay(100);                                    // mehr als 100 us...
  p4&=0xfe; LCD_E=1; delay(1); LCD_E=0;
  delay(100);                                    // mehr als 100 us...
  lcd_putbyte(0x28);                             // function set
  lcd_putbyte(0x08);                             // on/off-control
  lcd_putbyte(0x01);                             // Display loeschen
  delay(2000);                                   // mindestens 1,64 ms warten
  lcd_putbyte(0x06);                             // entry mode set
  lcd_putbyte(0x0c);                             // Display einschalten ohne Cursor
  lcd_putbyte(0x40);                             // CGRAM laden (alle acht Zeichen)
  LCD_RS=1;                                      // RS bleibt immer auf 1
  for (i=0;i<64;i++) lcd_putbyte(lcd_cgramcode[i]);
}

// Kursorpositionierung Spalte/Zeile
void lcd_gotoxy (unsigned char x, unsigned char y) {  // x = 0..15, y = 0/1
  if (y) x += 0x40;
  LCD_RS=0; lcd_putbyte(x | 0x80); LCD_RS=1;
}

// Textausgabe (bis delimiter ASCII-NUL)
void  lcd_puttext(unsigned char far *str) {
   unsigned char c;
   while (c= *str++) lcd_putbyte(c);
}

// Byte als zweistellige HEx-Zahl ausgeben 00..FF
void lcd_puthexc(unsigned char c) {
  unsigned char c1;
  c1=c/16;
  if(c1>9)  c1+= ('A'-10); else c1 += '0';       // 1.HEX-Ziffer aus oberem Nibbel
  lcd_putbyte(c1);
  c1=c%16;
  if(c1>9)  c1+= ('A'-10); else c1 += '0';       // 2.HEX-Ziffer aus oberem Nibbel
  lcd_putbyte(c1);
}

// Ausgabe vorzeichenlose Integer-Zahl (char, short, long) als Ziffernstring
// feste Feldweite field 1..9, freie fuehrende Felder mit Zeichen lead aufgefuellt
// ohne Kontrollen und Begrenzungen bis auf Feldweite
void lcd_putuint(unsigned long lval, unsigned char field, unsigned char lead) {
  unsigned long lhelp;
  if(field==0) field=1; if(field>9) field=9;     // Begrenzung 1..9
  for(lhelp=1;field>0;field--) lhelp*=10;        // lhelp 100.. entsprechend Feldweite
  if(lval>(lhelp-1)) lval=999999999;             // Ausgabe 99.. bei Feldueberlauf
  while(lhelp>1) {                               // Ziffer fuer Ziffer ausgeben
    lval=lval%lhelp;
    lhelp/=10;
    if(lhelp==1) lead='0';                       // letzte 0 immer ausgeben
    if(lval>=lhelp) lcd_putbyte(lval/lhelp+'0'), lead='0';
    else lcd_putbyte(lead);
} }

#define  lcd_light_on        p4_7=1
#define  lcd_light_off       p4_7=0


